@@ -22,10 +22,6 @@
 
 #include "kconfigini_p.h"
 
-#include <config.h>
-
-#include <fcntl.h>
-
 #include "kconfig.h"
 #include "kconfigbackend.h"
 #include "bufferfragment_p.h"
@@ -46,7 +42,7 @@ extern bool kde_kiosk_exception;
 
 QString KConfigIniBackend::warningProlog(const QFile &file, int line)
 {
-    return QString("KConfigIni: In file %2, line %1: ")
+    return QString::fromLatin1("KConfigIni: In file %2, line %1: ")
             .arg(line).arg(file.fileName());
 }
 
@@ -89,7 +85,7 @@ KConfigIniBackend::parseConfig(const QByteArray& currentLocale, KEntryMap& entry
     bool groupSkip = false;
 
     int lineNo = 0;
-    // on systems using \r\n as end of line, \r will be taken care of by 
+    // on systems using \r\n as end of line, \r will be taken care of by
     // trim() below
     QByteArray buffer = file.readAll();
     BufferFragment contents(buffer.data(), buffer.size());
@@ -230,9 +226,9 @@ KConfigIniBackend::parseConfig(const QByteArray& currentLocale, KEntryMap& entry
                             goto next_line; // skip this entry if we're not merging
                     }
                 }
-            } 
+            }
 
-            if (!(entryOptions & KEntryMap::EntryRawKey)) 
+            if (!(entryOptions & KEntryMap::EntryRawKey))
                 printableToString(&aKey, file, lineNo);
 
             if (options&ParseGlobal)
@@ -368,8 +364,9 @@ bool KConfigIniBackend::writeConfig(const QByteArray& locale, KEntryMap& entryMa
     Q_ASSERT(!filePath().isEmpty());
 
     KEntryMap writeMap;
-    bool bGlobal = options & WriteGlobal;
+    const bool bGlobal = options & WriteGlobal;
 
+    // First, reparse the file on disk, to merge our changes with the ones done by other apps
     {
         ParseOptions opts = ParseExpansions;
         if (bGlobal)
@@ -378,6 +375,7 @@ bool KConfigIniBackend::writeConfig(const QByteArray& locale, KEntryMap& entryMa
         if (info != ParseOk) // either there was an error or the file became immutable
             return false;
     }
+
     const KEntryMapIterator end = entryMap.end();
     for (KEntryMapIterator it=entryMap.begin(); it != end; ++it) {
         if (!it.key().mKey.isEmpty() && !it->bDirty) // not dirty, doesn't overwrite entry in writeMap. skips default entries, too.
@@ -392,10 +390,13 @@ bool KConfigIniBackend::writeConfig(const QByteArray& locale, KEntryMap& entryMa
             } else {
                 KEntryKey defaultKey = key;
                 defaultKey.bDefault = true;
-                if (!entryMap.contains(defaultKey))
+                if (!entryMap.contains(defaultKey)) {
                     writeMap.remove(key); // remove the deleted entry if there is no default
-                else
+                    //qDebug() << "Detected as deleted=>removed:" << key.mGroup << key.mKey << "global=" << bGlobal;
+                } else {
                     writeMap[key] = *it; // otherwise write an explicitly deleted entry
+                    //qDebug() << "Detected as deleted=>[$d]:" << key.mGroup << key.mKey << "global=" << bGlobal;
+                }
             }
             it->bDirty = false;
         }
@@ -435,6 +436,13 @@ bool KConfigIniBackend::writeConfig(const QByteArray& locale, KEntryMap& entryMa
         file.setTextModeEnabled(true); // to get eol translation
         writeEntries(locale, file, writeMap);
 
+        if (!file.flush()) {
+            // Couldn't write. Disk full?
+            kWarning() << "Couldn't write" << filePath() << ". Disk full?";
+            file.abort();
+            return false;
+        }
+
         if (!file.size() && (fileMode == (QFile::ReadUser | QFile::WriteUser))) {
             // File is empty and doesn't have special permissions: delete it.
             file.abort();
@@ -492,14 +500,18 @@ bool KConfigIniBackend::isWritable() const
         // The check might have failed because any of the containing dirs
         // did not exist. If the file does not exist, check if the deepest
         // existing dir is writable.
-        if (!QFileInfo(filePath()).exists()) {
-            QDir dir = QFileInfo(filePath()).absolutePath();
+        QFileInfo file(filePath());
+        if (!file.exists()) {
+            QFileInfo dir(file.absolutePath());
             while (!dir.exists()) {
-                if (!dir.cdUp()) {
+                QString parent = dir.absolutePath(); // Go up. Can't use cdUp() on non-existing dirs.
+                if (parent == dir.filePath()) {
+                    // no parent
                     return false;
                 }
+                dir.setFile(parent);
             }
-            return QFileInfo(dir.absolutePath()).isWritable();
+            return dir.isDir() && dir.isWritable();
         }
     }
 
@@ -529,8 +541,8 @@ void KConfigIniBackend::setFilePath(const QString& file)
 
     Q_ASSERT(QDir::isAbsolutePath(file));
 
-    if (QFile::exists(file)) {
-        const QFileInfo info(file);
+    const QFileInfo info(file);
+    if (info.exists()) {
         setLocalFilePath(info.canonicalFilePath());
         setLastModified(info.lastModified());
         setSize(info.size());
@@ -590,13 +602,16 @@ QByteArray KConfigIniBackend::stringToPrintable(const QByteArray& aString, Strin
     const int l = aString.length();
 
     QByteArray result; // Guesstimated that it's good to avoid data() initialization for a length of l*4
-    result.reserve(l * 4); // Maximum 4x as long as source string due to \x<ab> escape sequences
+    result.resize(l * 4); // Maximum 4x as long as source string due to \x<ab> escape sequences
     register const char *s = aString.constData();
     int i = 0;
+    char *data = result.data();
+    char *start = data;
 
     // Protect leading space
     if (s[0] == ' ' && type != GroupString) {
-        result.append("\\s");
+        *data++ = '\\';
+        *data++ = 's';
         i++;
     }
 
@@ -606,23 +621,27 @@ QByteArray KConfigIniBackend::stringToPrintable(const QByteArray& aString, Strin
             // The \n, \t, \r cases (all < 32) are handled below; we can ignore them here
                 if (((unsigned char)s[i]) < 32)
                     goto doEscape;
-                result.append(s[i]);
+                *data++ = s[i];
                 break;
             case '\n':
-                result.append("\\n");
+                *data++ = '\\';
+                *data++ = 'n';
                 break;
             case '\t':
-                result.append("\\t");
+                *data++ = '\\';
+                *data++ = 't';
                 break;
             case '\r':
-                result.append("\\r");
+                *data++ = '\\';
+                *data++ = 'r';
                 break;
             case '\\':
-                result.append("\\\\");
+                *data++ = '\\';
+                *data++ = '\\';
                 break;
             case '=':
                 if (type != KeyString) {
-                    result.append(s[i]);
+                    *data++ = s[i];
                     break;
                 }
                 goto doEscape;
@@ -630,16 +649,19 @@ QByteArray KConfigIniBackend::stringToPrintable(const QByteArray& aString, Strin
             case ']':
             // Above chars are OK to put in *value* strings as plaintext
                 if (type == ValueString) {
-                    result.append(s[i]);
+                    *data++ = s[i];
                     break;
                 }
         doEscape:
-                result.append("\\x")
-                    .append(nibbleLookup[((unsigned char)s[i]) >> 4])
-                    .append(nibbleLookup[((unsigned char)s[i]) & 0x0f]);
+                *data++ = '\\';
+                *data++ = 'x';
+                *data++ = nibbleLookup[((unsigned char)s[i]) >> 4];
+                *data++ = nibbleLookup[((unsigned char)s[i]) & 0x0f];
                 break;
         }
     }
+    *data = 0;
+    result.resize(data - start);
 
     // Protect trailing space
     if (result.endsWith(' ') && type != GroupString) {
@@ -676,7 +698,7 @@ char KConfigIniBackend::charFromHex(const char *str, const QFile& file, int line
 
 void KConfigIniBackend::printableToString(BufferFragment* aString, const QFile& file, int line)
 {
-    if (aString->isEmpty() || aString->indexOf('\\')==-1) 
+    if (aString->isEmpty() || aString->indexOf('\\')==-1)
         return;
     aString->trim();
     int l = aString->length();
@@ -722,7 +744,7 @@ void KConfigIniBackend::printableToString(BufferFragment* aString, const QFile&
                 default:
                     *r = '\\';
                     qWarning() << warningProlog(file, line)
-                            << QString("Invalid escape sequence \"\\%1\".").arg(str[i]);
+                               << QString::fromLatin1("Invalid escape sequence \"\\%1\".").arg(str[i]);
             }
         }
     }
