@@ -23,6 +23,7 @@
 #include <kconfiggroup.h>
 #include <qtest_kde.h>
 #include <kdebug.h>
+#include <kprocess.h>
 #include <kstandarddirs.h>
 #include "kdebug_unittest.moc"
 
@@ -30,6 +31,9 @@ QTEST_KDEMAIN_CORE( KDebugTest )
 
 void KDebugTest::initTestCase()
 {
+    setenv("KDE_DEBUG_FILELINE", "", 1);
+    setenv("KDE_DEBUG_TIMESTAMP", "", 1);
+
     QString kdebugrc = KStandardDirs::locateLocal("config", "kdebugrc");
     if (!kdebugrc.isEmpty())
         QFile::remove(kdebugrc);
@@ -45,9 +49,11 @@ void KDebugTest::initTestCase()
     config.group("qttest").writeEntry("WarnOutput", 0 /*FileOutput*/);
     config.sync();
 
+    //QCOMPARE(KDebug::hasNullOutput(QtDebugMsg, true, 0, false), false);
+
     // Test for crash that used to happen when using an unknown area after only dynamic areas
-    KDebug::registerArea("somearea");
-    KDebug::registerArea("someotherarea");
+    KDebug::registerArea("somearea"); // gets number 1
+    KDebug::registerArea("someotherarea"); // gets number 2
     QCOMPARE(KDebug::hasNullOutput(QtDebugMsg, true, 4242, false), false); // unknown area -> area 0 is being used
 
     kClearDebugConfig();
@@ -86,10 +92,17 @@ void KDebugTest::compareLines(const QList<QByteArray>& expectedLines, const char
     QList<QByteArray> lines = readLines(fileName);
     //qDebug() << lines;
     QCOMPARE(lines.count(), expectedLines.count());
-    QVERIFY(lines[0].endsWith("\n"));
+    QVERIFY(lines[0].endsWith('\n'));
     for (int i = 0; i < lines.count(); ++i) {
-        //qDebug() << "line" << i << ":" << lines[i] << expectedLines[i];
-        QVERIFY(lines[i].endsWith(expectedLines[i]));
+        QByteArray line = lines[i];
+        if (expectedLines[i].contains("[...]")) {
+            const int pos = line.indexOf('[');
+            QVERIFY(pos >= 0);
+            line.truncate(pos);
+            line.append("[...]\n");
+        }
+        //qDebug() << "line" << i << ":" << line << expectedLines[i];
+        QVERIFY(line.endsWith(expectedLines[i]));
     }
 }
 
@@ -125,6 +138,12 @@ void KDebugTest::testDebugToFile()
     kDebug(0) << "TEST DEBUG with newline" << endl << "newline";
     TestClass tc;
     kDebug(0) << "Re-entrance test" << tc << "[ok]";
+    {
+        KDebug::Block block("block 1");
+        {
+            KDebug::Block block("block 2");
+        }
+    }
     QVERIFY(QFile::exists("kdebug.dbg"));
     QList<QByteArray> expected;
     expected << "/kdecore (kdelibs) KDebugTest::testDebugToFile: TEST DEBUG 180\n";
@@ -134,6 +153,10 @@ void KDebugTest::testDebugToFile()
     expected << "newline\n";
     expected << "/kdecore (kdelibs) TestClass::getSomething: Nested kDebug call\n";
     expected << "Re-entrance test \"TestClass\" after the call [ok]\n";
+    expected << "BEGIN: block 1\n";
+    expected << "BEGIN: block 2\n";
+    expected << "END__: block 2 [...]\n";
+    expected << "END__: block 1 [...]\n";
     compareLines(expected);
 }
 
@@ -147,6 +170,10 @@ void KDebugTest::testDisableArea()
     kClearDebugConfig();
     kDebug(180) << "TEST DEBUG 180 - SHOULD NOT APPEAR";
     kDebug(0) << "TEST DEBUG 0 - SHOULD NOT APPEAR";
+    {
+        KDebug::Block block("SHOULD NOT APPEAR");
+        kDebug(0) << "msg inside the block, should not appear";
+    }
     QVERIFY(!QFile::exists("kdebug.dbg"));
 
     // Re-enable debug, for further tests
@@ -158,14 +185,14 @@ void KDebugTest::testDisableArea()
 
 void KDebugTest::testDynamicArea()
 {
-    const int myArea = KDebug::registerArea("myarea");
-    QVERIFY(myArea > 0);
+    const int myArea = KDebug::registerArea("myarea"); // gets number 3
+    QCOMPARE(myArea, 3);
     KConfig config("kdebugrc");
     QVERIFY(!config.hasGroup(QString::number(myArea)));
     QVERIFY(config.hasGroup("myarea"));
     kDebug(myArea) << "TEST DEBUG using myArea" << myArea;
     QList<QByteArray> expected;
-    expected << "/myarea KDebugTest::testDynamicArea: TEST DEBUG using myArea 1\n";
+    expected << "/myarea KDebugTest::testDynamicArea: TEST DEBUG using myArea 3\n";
     compareLines(expected, "myarea.dbg");
 }
 
@@ -231,6 +258,42 @@ void KDebugTest::testHasNullOutput()
     kClearDebugConfig();
 }
 
+void KDebugTest::testNoMainComponentData()
+{
+    // This test runs kdebug_qcoreapptest and checks its output
+    KProcess proc;
+    proc.setEnv("KDE_DEBUG_NOPROCESSINFO", "1");
+    proc.setEnv("KDE_DEBUG_TIMESTAMP", "0");
+    proc.setOutputChannelMode(KProcess::OnlyStderrChannel);
+#ifdef Q_OS_WIN
+    proc << "kdebug_qcoreapptest.exe";
+#else
+    if (QFile::exists("./kdebug_qcoreapptest.shell"))
+        proc << "./kdebug_qcoreapptest.shell";
+    else {
+        QVERIFY(QFile::exists("./kdebug_qcoreapptest"));
+        proc << "./kdebug_qcoreapptest";
+    }
+#endif
+    //     kDebug() << proc.args();
+    const int ok = proc.execute();
+    QVERIFY(ok == 0);
+    const QByteArray allOutput = proc.readAllStandardError();
+    const QList<QByteArray> receivedLines = allOutput.split('\n');
+    //qDebug() << receivedLines;
+    QList<QByteArray> expectedLines;
+    expectedLines << "qcoreapp_myarea main: Test debug using qcoreapp_myarea 1";
+    expectedLines << "kdebug_qcoreapptest main: Debug in area 100";
+    expectedLines << "kdebug_qcoreapptest main: Simple debug";
+    expectedLines << "kdebug_qcoreapptest_mainData main: This should appear, under the kdebug_qcoreapptest_mainData area";
+    expectedLines << "kdebug_qcoreapptest_mainData main: Debug in area 100";
+    expectedLines << ""; // artefact of split, I guess?
+    for (int i = 0; i < qMin(expectedLines.count(), receivedLines.count()); ++i)
+        QCOMPARE(QString::fromLatin1(receivedLines[i]), QString::fromLatin1(expectedLines[i]));
+    QCOMPARE(receivedLines.count(), expectedLines.count());
+    QCOMPARE(receivedLines, expectedLines);
+}
+
 #include <QThreadPool>
 #include <qtconcurrentrun.h>
 
@@ -239,6 +302,7 @@ class KDebugThreadTester
 public:
     void doDebugs()
     {
+        KDEBUG_BLOCK
         for (int i = 0; i < 10; ++i)
             kDebug() << "A kdebug statement in a thread:" << i;
     }
@@ -255,22 +319,27 @@ void KDebugTest::testMultipleThreads()
     QList<QFuture<void> > futures;
     for (int threadNum = 0; threadNum < 10; ++threadNum)
         futures << QtConcurrent::run(&tester, &KDebugThreadTester::doDebugs);
-    Q_FOREACH(QFuture<void> f, futures)
+    Q_FOREACH(QFuture<void> f, futures) // krazy:exclude=foreach
         f.waitForFinished();
 
     QVERIFY(QFile::exists("kdebug.dbg"));
 
+    //QFile f("kdebug.dbg"); f.open(QIODevice::ReadOnly);
+    //qDebug() << QString::fromLatin1(f.readAll());
+
     // We have no guarantee that the debug lines are issued one after the other.
     // The \n comes from the destruction of the temp kDebug, and that's not mutexed,
     // so we can get msg1 + msg2 + \n + \n.
     // So this test is basically only good for running in helgrind.
+
 #if 0
     // Check that the lines are whole
     QList<QByteArray> lines = readLines();
     Q_FOREACH(const QByteArray& line, lines) {
-        //qDebug() << line;
-        QCOMPARE(line.count("doDebugs: A kdebug statement in a thread:"), 1);
+        qDebug() << line;
+        QCOMPARE(line.count("doDebugs"), 1);
         QCOMPARE(line.count('\n'), 1);
+        QVERIFY(!line.startsWith("   ")); // more than 2 spaces? indentString messed up
     }
 #endif
 }
