@@ -26,7 +26,7 @@
 
 #include <windows.h>
 #include <lm.h>
-#include <Sddl.h>
+#include <sddl.h>
 
 class KUser::Private : public KShared
 {
@@ -46,7 +46,7 @@ class KUser::Private : public KShared
             {
                 servername = NULL;
             }
-            
+
             if (NetUserGetInfo((LPCWSTR) servername, (LPCWSTR) name.utf16(), 11, (LPBYTE *) &userInfo) != NERR_Success) {
                 goto error;
             }
@@ -61,7 +61,7 @@ class KUser::Private : public KShared
                 SID_NAME_USE nameuse;
                 DWORD cchReferencedDomainName = 0;
                 WCHAR* referencedDomainName = NULL;
-                
+
                 // the following line definitely fails:
                 // both the sizes for sid and for referencedDomainName are Null
                 // the needed sizes are set in size and cchReferencedDomainName
@@ -102,7 +102,7 @@ class KUser::Private : public KShared
                 NetApiBufferFree(userInfo);
                 userInfo = 0;
             }
-            if (servername) 
+            if (servername)
             {
                 NetApiBufferFree(servername);
                 servername = 0;
@@ -189,14 +189,16 @@ QString KUser::loginName() const
     return (d->userInfo ? QString::fromUtf16((ushort *) d->userInfo->usri11_name) : QString());
 }
 
+#ifndef KDE_NO_DEPRECATED
 QString KUser::fullName() const
 {
     return (d->userInfo ? QString::fromUtf16((ushort *) d->userInfo->usri11_full_name) : QString());
 }
+#endif
 
 QString KUser::homeDir() const
 {
-	return QDir::fromNativeSeparators(qgetenv("USERPROFILE"));
+	return QDir::fromNativeSeparators(QString::fromLocal8Bit(qgetenv("USERPROFILE")));
 }
 
 QString KUser::faceIconPath() const
@@ -256,7 +258,7 @@ K_UID KUser::uid() const
 
 QVariant KUser::property(UserProperty which) const
 {
-    if (which == FullName) 
+    if (which == FullName)
         return QVariant(d->userInfo ? QString::fromUtf16((ushort *) d->userInfo->usri11_full_name) : QString());
 
     return QVariant();
@@ -340,7 +342,7 @@ KUserGroup::KUserGroup(const QString &_name)
 }
 
 KUserGroup::KUserGroup(const char *_name)
-    : d(new Private(QString(_name)))
+    : d(new Private(QLatin1String(_name)))
 {
 }
 
@@ -375,7 +377,9 @@ bool KUserGroup::isValid() const
 
 QString KUserGroup::name() const
 {
-    return QString::fromUtf16((ushort *) d->groupInfo->grpi0_name);
+    if(d && d->groupInfo)
+        return QString::fromUtf16((ushort *) d->groupInfo->grpi0_name);
+    return QString();
 }
 
 QList<KUser> KUserGroup::users() const
@@ -430,7 +434,7 @@ QList<KUserGroup> KUserGroup::allGroups()
     KUserGroup tmp("");
 
     do {
-        nStatus = NetGroupEnum(NULL, 0, (LPBYTE*) &pGroup, 1, &dwEntriesRead, &dwTotalEntries, &dwResumeHandle);
+        nStatus = NetGroupEnum(NULL, 0, (LPBYTE*) &pGroup, 1, &dwEntriesRead, &dwTotalEntries, (PDWORD_PTR)&dwResumeHandle);
 
         if ((nStatus == NERR_Success || nStatus == ERROR_MORE_DATA) && dwEntriesRead > 0) {
             tmp.d = new Private(pGroup);
