@@ -92,6 +92,7 @@ public:
     {
 // causes a weird crash in KWord at least, so let Qt delete it for us.
 //        delete completionBox;
+        delete style.data();
     }
 
     void _k_slotSettingsChanged(int category)
@@ -109,7 +110,9 @@ public:
         // KDE5: remove userTextChanged signal, textEdited does the same...
         if (!completionRunning && (txt != userText)) {
             userText = txt;
+#ifndef KDE_NO_DEPRECATED
             emit q->userTextChanged(txt);
+#endif
         }
     }
 
@@ -120,7 +123,9 @@ public:
         if (!completionRunning && (txt != userText)) {
             userText = txt;
             q->setModified(true);
+#ifndef KDE_NO_DEPRECATED
             emit q->userTextChanged(txt);
+#endif
             emit q->textEdited(txt);
             emit q->textChanged(txt);
         }
@@ -271,7 +276,6 @@ void KLineEdit::init()
     setStyle(d->style.data());
 
     connect(this, SIGNAL(textChanged(QString)), this, SLOT(_k_textChanged(QString)));
-
 }
 
 QString KLineEdit::clickMessage() const
@@ -287,6 +291,7 @@ void KLineEdit::setClearButtonShown(bool show)
         }
 
         d->clearButton = new KLineEditButton(this);
+        d->clearButton->setObjectName("KLineEditButton");
         d->clearButton->setCursor( Qt::ArrowCursor );
         d->clearButton->setToolTip( i18nc( "@action:button Clear current text in the line edit", "Clear text" ) );
 
@@ -793,6 +798,31 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
     if ( echoMode() == QLineEdit::Normal &&
          completionMode() != KGlobalSettings::CompletionNone )
     {
+        if (e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter) {
+            const bool trap = (d->completionBox && d->completionBox->isVisible());
+            const bool stopEvent = (trap || (d->grabReturnKeyEvents &&
+                                    (e->modifiers() == Qt::NoButton ||
+                                     e->modifiers() == Qt::KeypadModifier)));
+
+            if (stopEvent) {
+                emit QLineEdit::returnPressed();
+                e->accept();
+            }
+
+            emit returnPressed( displayText() );
+
+            if (trap) {
+                d->completionBox->hide();
+                deselect();
+                setCursorPosition(text().length());
+            }
+
+            // Eat the event if the user asked for it, or if a completionbox was visible
+            if (stopEvent) {
+                return;
+            }
+        }
+
         const KeyBindingMap keys = getKeyBindings();
         const KGlobalSettings::Completion mode = completionMode();
         const bool noModifier = (e->modifiers() == Qt::NoButton ||
@@ -1090,7 +1120,7 @@ void KLineEdit::mousePressEvent( QMouseEvent* e )
     if  ( (e->button() == Qt::LeftButton ||
            e->button() == Qt::MidButton ) &&
           d->clearButton ) {
-        d->clickInClear = d->clearButton->underMouse();
+        d->clickInClear = ( d->clearButton == childAt(e->pos()) || d->clearButton->underMouse() );
 
         if ( d->clickInClear ) {
             d->possibleTripleClick = false;
@@ -1103,13 +1133,23 @@ void KLineEdit::mousePressEvent( QMouseEvent* e )
         return;
     }
 
+    // if middle clicking and if text is present in the clipboard then clear the selection
+    // to prepare paste operation
+    if ( e->button() == Qt::MidButton ) {
+        if ( hasSelectedText() ) {
+            if ( QApplication::clipboard()->text( QClipboard::Selection ).length() >0 ) {
+                backspace();
+            }
+        }
+    }
+
     QLineEdit::mousePressEvent( e );
 }
 
 void KLineEdit::mouseReleaseEvent( QMouseEvent* e )
 {
     if ( d->clickInClear ) {
-        if ( d->clearButton->underMouse() ) {
+        if ( d->clearButton == childAt(e->pos()) || d->clearButton->underMouse() ) {
             QString newText;
             if ( e->button() == Qt::MidButton ) {
                 newText = QApplication::clipboard()->text( QClipboard::Selection );
@@ -1129,12 +1169,12 @@ void KLineEdit::mouseReleaseEvent( QMouseEvent* e )
 
     QLineEdit::mouseReleaseEvent( e );
 
-   if (QApplication::clipboard()->supportsSelection() ) {
-       if ( e->button() == Qt::LeftButton ) {
+    if (QApplication::clipboard()->supportsSelection() ) {
+        if ( e->button() == Qt::LeftButton ) {
             // Fix copying of squeezed text if needed
             copySqueezedText( false );
-       }
-   }
+        }
+    }
 }
 
 void KLineEdit::tripleClickTimeout()
@@ -1145,9 +1185,6 @@ void KLineEdit::tripleClickTimeout()
 QMenu* KLineEdit::createStandardContextMenu()
 {
     QMenu *popup = QLineEdit::createStandardContextMenu();
-    if ( !popup) { // happens on some embedded platforms
-        return 0;
-    }
 
     if( !isReadOnly() )
     {
@@ -1162,7 +1199,7 @@ QMenu* KLineEdit::createStandardContextMenu()
             separatorAction = actionList.at( idx );
         if ( separatorAction )
         {
-            KAction *clearAllAction = KStandardAction::clear( this, SLOT( clear() ), this) ;
+            KAction *clearAllAction = KStandardAction::clear( this, SLOT(clear()), this) ;
             if ( text().isEmpty() )
                 clearAllAction->setEnabled( false );
             popup->insertAction( separatorAction, clearAllAction );
@@ -1177,8 +1214,8 @@ QMenu* KLineEdit::createStandardContextMenu()
     if ( compObj() && !isReadOnly() && KAuthorized::authorize("lineedit_text_completion") )
     {
         QMenu *subMenu = popup->addMenu( KIcon("text-completion"), i18nc("@title:menu", "Text Completion") );
-        connect( subMenu, SIGNAL( triggered ( QAction* ) ),
-                 this, SLOT( completionMenuActivated( QAction* ) ) );
+        connect( subMenu, SIGNAL(triggered(QAction*)),
+                 this, SLOT(completionMenuActivated(QAction*)) );
 
         popup->addSeparator();
 
@@ -1332,37 +1369,6 @@ bool KLineEdit::event( QEvent* ev )
         if (d->overrideShortcut(e)) {
             ev->accept();
         }
-    } else if( ev->type() == QEvent::KeyPress ) {
-        // Hmm -- all this could be done in keyPressEvent too...
-
-        QKeyEvent *e = static_cast<QKeyEvent *>( ev );
-
-        if (e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter) {
-            const bool trap = d->completionBox && d->completionBox->isVisible();
-
-            const bool stopEvent = trap || (d->grabReturnKeyEvents &&
-                                      (e->modifiers() == Qt::NoButton ||
-                                       e->modifiers() == Qt::KeypadModifier));
-
-            // Qt will emit returnPressed() itself if we return false
-            if (stopEvent) {
-                emit QLineEdit::returnPressed();
-                e->accept();
-            }
-
-            emit returnPressed( displayText() );
-
-            if (trap) {
-                d->completionBox->hide();
-                deselect();
-                setCursorPosition(text().length());
-            }
-
-            // Eat the event if the user asked for it, or if a completionbox was visible
-            if (stopEvent) {
-                return true;
-            }
-        }
     } else if (ev->type() == QEvent::ApplicationPaletteChange
                || ev->type() == QEvent::PaletteChange) {
         // Assume the widget uses the application's palette
@@ -1431,21 +1437,37 @@ void KLineEdit::setCompletionBox( KCompletionBox *box )
     d->completionBox = box;
     if ( handleSignals() )
     {
-        connect( d->completionBox, SIGNAL(currentTextChanged( const QString& )),
-                 SLOT(_k_slotCompletionBoxTextChanged( const QString& )) );
-        connect( d->completionBox, SIGNAL(userCancelled( const QString& )),
-                 SLOT(userCancelled( const QString& )) );
+        connect( d->completionBox, SIGNAL(currentTextChanged(QString)),
+                 SLOT(_k_slotCompletionBoxTextChanged(QString)) );
+        connect( d->completionBox, SIGNAL(userCancelled(QString)),
+                 SLOT(userCancelled(QString)) );
         connect( d->completionBox, SIGNAL(activated(QString)),
                  SIGNAL(completionBoxActivated(QString)) );
+        connect( d->completionBox, SIGNAL(activated(QString)),
+                 SIGNAL(textEdited(QString)) );
     }
 }
 
+/*
+ * Set the line edit text without changing the modified flag. By default
+ * calling setText resets the modified flag to false.
+ */
+static void setEditText(KLineEdit* edit, const QString& text)
+{
+    if (!edit) {
+        return;
+    }
+
+    const bool wasModified = edit->isModified();
+    edit->setText(text);
+    edit->setModified(wasModified);
+}
+
 void KLineEdit::userCancelled(const QString & cancelText)
 {
     if ( completionMode() != KGlobalSettings::CompletionPopupAuto )
     {
-      // TODO: this sets modified==false. But maybe it was true before...
-      setText(cancelText);
+      setEditText(this, cancelText);
     }
     else if (hasSelectedText() )
     {
@@ -1455,8 +1477,8 @@ void KLineEdit::userCancelled(const QString & cancelText)
       {
         d->autoSuggest=false;
         const int start = selectionStart() ;
-        const QString s=text().remove(selectionStart(), selectedText().length());
-        setText(s);
+        const QString s = text().remove(selectionStart(), selectedText().length());
+        setEditText(this, s);
         setCursorPosition(start);
         d->autoSuggest=true;
       }
@@ -1620,12 +1642,12 @@ void KLineEdit::setCompletionObject( KCompletion* comp, bool hsig )
 {
     KCompletion *oldComp = compObj();
     if ( oldComp && handleSignals() )
-        disconnect( oldComp, SIGNAL( matches( const QStringList& )),
-                    this, SLOT( setCompletedItems( const QStringList& )));
+        disconnect( oldComp, SIGNAL(matches(QStringList)),
+                    this, SLOT(setCompletedItems(QStringList)));
 
     if ( comp && hsig )
-      connect( comp, SIGNAL( matches( const QStringList& )),
-               this, SLOT( setCompletedItems( const QStringList& )));
+      connect( comp, SIGNAL(matches(QStringList)),
+               this, SLOT(setCompletedItems(QStringList)));
 
     KCompletionBase::setCompletionObject( comp, hsig );
 }
@@ -1685,7 +1707,6 @@ void KLineEdit::_k_slotCompletionBoxTextChanged( const QString& text )
     {
         setText( text );
         setModified(true);
-        emit textEdited(text);
         end( false ); // force cursor at end
     }
 }
@@ -1810,15 +1831,19 @@ void KLineEdit::setClickMessage( const QString &msg )
     update();
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KLineEdit::setContextMenuEnabled( bool showMenu )
 {
     QLineEdit::setContextMenuPolicy( showMenu ? Qt::DefaultContextMenu : Qt::NoContextMenu );
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 bool KLineEdit::isContextMenuEnabled() const
 {
     return  ( contextMenuPolicy() == Qt::DefaultContextMenu );
 }
+#endif
 
 void KLineEdit::setPasswordMode(bool b)
 {
@@ -1858,4 +1883,3 @@ void KLineEdit::doCompletion(const QString& txt)
 
 #include "klineedit.moc"
 #include "klineedit_p.moc"
-
