@@ -32,30 +32,28 @@ class KSqueezedTextLabelPrivate
 public:
 
     void _k_copyFullText() {
-        QMimeData* data = new QMimeData;
-        data->setText(fullText);
-        QApplication::clipboard()->setMimeData(data);
+        QApplication::clipboard()->setText(fullText);
     }
 
     QString fullText;
     Qt::TextElideMode elideMode;
 };
 
-KSqueezedTextLabel::KSqueezedTextLabel( const QString &text , QWidget *parent )
- : QLabel ( parent ),
-  d( new KSqueezedTextLabelPrivate )
+KSqueezedTextLabel::KSqueezedTextLabel(const QString &text , QWidget *parent)
+ : QLabel (parent),
+  d(new KSqueezedTextLabelPrivate)
 {
-  setSizePolicy(QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ));
+  setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
   d->fullText = text;
   d->elideMode = Qt::ElideMiddle;
   squeezeTextToLabel();
 }
 
-KSqueezedTextLabel::KSqueezedTextLabel( QWidget *parent )
- : QLabel ( parent ),
-  d( new KSqueezedTextLabelPrivate )
+KSqueezedTextLabel::KSqueezedTextLabel(QWidget *parent)
+ : QLabel (parent),
+  d(new KSqueezedTextLabelPrivate)
 {
-  setSizePolicy(QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ));
+  setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
   d->elideMode = Qt::ElideMiddle;
 }
 
@@ -64,7 +62,8 @@ KSqueezedTextLabel::~KSqueezedTextLabel()
   delete d;
 }
 
-void KSqueezedTextLabel::resizeEvent( QResizeEvent * ) {
+void KSqueezedTextLabel::resizeEvent(QResizeEvent *)
+{
   squeezeTextToLabel();
 }
 
@@ -77,7 +76,7 @@ QSize KSqueezedTextLabel::minimumSizeHint() const
 
 QSize KSqueezedTextLabel::sizeHint() const
 {
-  int maxWidth = KGlobalSettings::desktopGeometry( this ).width() * 3 / 4;
+  int maxWidth = KGlobalSettings::desktopGeometry(this).width() * 3 / 4;
   QFontMetrics fm(fontMetrics());
   int textWidth = fm.width(d->fullText);
   if (textWidth > maxWidth) {
@@ -86,17 +85,20 @@ QSize KSqueezedTextLabel::sizeHint() const
   return QSize(textWidth, QLabel::sizeHint().height());
 }
 
-void KSqueezedTextLabel::setText( const QString &text ) {
+void KSqueezedTextLabel::setText(const QString &text)
+{
   d->fullText = text;
   squeezeTextToLabel();
 }
 
-void KSqueezedTextLabel::clear() {
+void KSqueezedTextLabel::clear()
+{
   d->fullText.clear();
   QLabel::clear();
 }
 
-void KSqueezedTextLabel::squeezeTextToLabel() {
+void KSqueezedTextLabel::squeezeTextToLabel()
+{
   QFontMetrics fm(fontMetrics());
   int labelWidth = size().width();
   QStringList squeezedLines;
@@ -116,11 +118,11 @@ void KSqueezedTextLabel::squeezeTextToLabel() {
     setToolTip(d->fullText);
   } else {
     QLabel::setText(d->fullText);
-    setToolTip( QString() );
+    setToolTip(QString());
   }
 }
 
-void KSqueezedTextLabel::setAlignment( Qt::Alignment alignment )
+void KSqueezedTextLabel::setAlignment(Qt::Alignment alignment)
 {
   // save fullText and restore it
   QString tmpFull(d->fullText);
@@ -171,4 +173,33 @@ void KSqueezedTextLabel::contextMenuEvent(QContextMenuEvent* ev)
     }
 }
 
+void KSqueezedTextLabel::mouseReleaseEvent(QMouseEvent* ev)
+{
+#if QT_VERSION >= 0x040700
+    if (QApplication::clipboard()->supportsSelection() &&
+        textInteractionFlags() != Qt::NoTextInteraction &&
+        ev->button() == Qt::LeftButton &&
+        !d->fullText.isEmpty() &&
+        hasSelectedText()) {
+        // Expand "..." when selecting with the mouse
+        QString txt = selectedText();
+        const QChar ellipsisChar(0x2026); // from qtextengine.cpp
+        const int dotsPos = txt.indexOf(ellipsisChar);
+        if (dotsPos > -1) {
+            // Ex: abcde...yz, selecting de...y  (selectionStart=3)
+            // charsBeforeSelection = selectionStart = 2 (ab)
+            // charsAfterSelection = 1 (z)
+            // final selection length= 26 - 2 - 1 = 23
+            const int start = selectionStart();
+            const int charsAfterSelection = text().length() - start - selectedText().length();
+            txt = d->fullText.mid(selectionStart(), d->fullText.length() - start - charsAfterSelection);
+        }
+        QApplication::clipboard()->setText(txt, QClipboard::Selection);
+    } else
+#endif
+    {
+        QLabel::mouseReleaseEvent(ev);
+    }
+}
+
 #include "ksqueezedtextlabel.moc"
