@@ -122,8 +122,11 @@ KFilePlacesModel::KFilePlacesModel(QObject *parent)
 #ifdef Q_OS_WIN
         // adding drives
         foreach ( const QFileInfo& info, QDir::drives() ) {
+#ifndef _WIN32_WCE
             uint type = DRIVE_UNKNOWN;
+#endif
             QString driveIcon = "drive-harddisk";
+#ifndef _WIN32_WCE
             QT_WA({ type = GetDriveTypeW((wchar_t *)info.absoluteFilePath().utf16()); },
                   { type = GetDriveTypeA(info.absoluteFilePath().toLocal8Bit()); });
             // qDebug() << "drive " << info.absoluteFilePath() << " type: " << type;
@@ -146,6 +149,7 @@ KFilePlacesModel::KFilePlacesModel(QObject *parent)
                 default:
                     driveIcon = "drive-harddisk";
             }
+#endif
             KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                                   info.absoluteFilePath(), info.absoluteFilePath(),
                                                   KUrl(info.absoluteFilePath()), driveIcon);
@@ -171,16 +175,18 @@ KFilePlacesModel::KFilePlacesModel(QObject *parent)
     d->sharedBookmarks = new KFilePlacesSharedBookmarks(d->bookmarkManager);
 
     d->predicate = Solid::Predicate::fromString(
-        "[[[ StorageVolume.ignored == false AND [ StorageVolume.usage == 'FileSystem' OR StorageVolume.usage == 'Encrypted' ]]"
+        "[[[[ StorageVolume.ignored == false AND [ StorageVolume.usage == 'FileSystem' OR StorageVolume.usage == 'Encrypted' ]]"
         " OR "
         "[ IS StorageAccess AND StorageDrive.driveType == 'Floppy' ]]"
         " OR "
-        "OpticalDisc.availableContent & 'Audio' ]");
+        "OpticalDisc.availableContent & 'Audio' ]"
+        " OR "
+        "StorageAccess.ignored == false ]");
     Q_ASSERT(d->predicate.isValid());
 
-    connect(d->bookmarkManager, SIGNAL(changed(const QString&, const QString&)),
+    connect(d->bookmarkManager, SIGNAL(changed(QString,QString)),
             this, SLOT(_k_reloadBookmarks()));
-    connect(d->bookmarkManager, SIGNAL(bookmarksChanged(const QString&)),
+    connect(d->bookmarkManager, SIGNAL(bookmarksChanged(QString)),
             this, SLOT(_k_reloadBookmarks()));
 
     d->_k_reloadBookmarks();
@@ -327,10 +333,10 @@ void KFilePlacesModel::Private::_k_initDeviceList()
 {
     Solid::DeviceNotifier *notifier = Solid::DeviceNotifier::instance();
 
-    connect(notifier, SIGNAL(deviceAdded(const QString&)),
-            q, SLOT(_k_deviceAdded(const QString&)));
-    connect(notifier, SIGNAL(deviceRemoved(const QString&)),
-            q, SLOT(_k_deviceRemoved(const QString&)));
+    connect(notifier, SIGNAL(deviceAdded(QString)),
+            q, SLOT(_k_deviceAdded(QString)));
+    connect(notifier, SIGNAL(deviceRemoved(QString)),
+            q, SLOT(_k_deviceRemoved(QString)));
 
     const QList<Solid::Device> &deviceList = Solid::Device::listFromQuery(predicate);
 
@@ -464,8 +470,8 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
             } else {
                 item = new KFilePlacesItem(bookmarkManager, bookmark.address());
             }
-            connect(item, SIGNAL(itemChanged(const QString&)),
-                    q, SLOT(_k_itemChanged(const QString&)));
+            connect(item, SIGNAL(itemChanged(QString)),
+                    q, SLOT(_k_itemChanged(QString)));
             items << item;
         }
 
@@ -478,8 +484,8 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
         if (!bookmark.isNull()) {
             KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager,
                                                         bookmark.address(), udi);
-            connect(item, SIGNAL(itemChanged(const QString&)),
-                    q, SLOT(_k_itemChanged(const QString&)));
+            connect(item, SIGNAL(itemChanged(QString)),
+                    q, SLOT(_k_itemChanged(QString)));
             // TODO: Update bookmark internal element
             items << item;
         }
@@ -603,6 +609,7 @@ bool KFilePlacesModel::dropMimeData(const QMimeData *data, Qt::DropAction action
         KBookmarkGroup group = d->bookmarkManager->root();
 
         foreach (const KUrl &url, urls) {
+            // TODO: use KIO::stat in order to get the UDS_DISPLAY_NAME too
             KMimeType::Ptr mimetype = KMimeType::mimeType(KIO::NetAccess::mimetype(url, 0));
 
             if (!mimetype) {
@@ -791,8 +798,8 @@ void KFilePlacesModel::requestTeardown(const QModelIndex &index)
     Solid::StorageAccess *access = device.as<Solid::StorageAccess>();
 
     if (access!=0) {
-        connect(access, SIGNAL(teardownDone(Solid::ErrorType, QVariant, const QString &)),
-                this, SLOT(_k_storageTeardownDone(Solid::ErrorType, QVariant)));
+        connect(access, SIGNAL(teardownDone(Solid::ErrorType,QVariant,QString)),
+                this, SLOT(_k_storageTeardownDone(Solid::ErrorType,QVariant)));
 
         access->teardown();
     }
@@ -805,8 +812,8 @@ void KFilePlacesModel::requestEject(const QModelIndex &index)
     Solid::OpticalDrive *drive = device.parent().as<Solid::OpticalDrive>();
 
     if (drive!=0) {
-        connect(drive, SIGNAL(ejectDone(Solid::ErrorType, QVariant, const QString &)),
-                this, SLOT(_k_storageTeardownDone(Solid::ErrorType, QVariant)));
+        connect(drive, SIGNAL(ejectDone(Solid::ErrorType,QVariant,QString)),
+                this, SLOT(_k_storageTeardownDone(Solid::ErrorType,QVariant)));
 
         drive->eject();
     } else {
@@ -828,8 +835,8 @@ void KFilePlacesModel::requestSetup(const QModelIndex &index)
 
         d->setupInProgress[access] = index;
 
-        connect(access, SIGNAL(setupDone(Solid::ErrorType, QVariant, const QString &)),
-                this, SLOT(_k_storageSetupDone(Solid::ErrorType, QVariant)));
+        connect(access, SIGNAL(setupDone(Solid::ErrorType,QVariant,QString)),
+                this, SLOT(_k_storageSetupDone(Solid::ErrorType,QVariant)));
 
         access->setup();
     }
