@@ -1,6 +1,6 @@
 /*
  * This file is part of the Nepomuk KDE project.
- * Copyright (C) 2006-2009 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2006-2010 Sebastian Trueg <trueg@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,14 +26,17 @@
 #include "tag.h"
 #include "pimo.h"
 #include "thing.h"
+#include "file.h"
 #include "property.h"
 #include "nfo.h"
 #include "nie.h"
 #include "nco.h"
 #include "nuao.h"
 
-#include <klocale.h>
-#include <kdebug.h>
+#include "klocale.h"
+#include "kdebug.h"
+#include "kurl.h"
+#include "kmimetype.h"
 
 #include <Soprano/Vocabulary/NAO>
 #include <Soprano/Vocabulary/Xesam>
@@ -41,11 +44,10 @@
 #include <Soprano/Model>
 #include <Soprano/QueryResultIterator>
 
-#include <kmimetype.h>
-
 
 Nepomuk::Resource::Resource()
 {
+    QMutexLocker lock( &ResourceManager::instance()->d->mutex );
     m_data = ResourceManager::instance()->d->data( QUrl(), QUrl() );
     if ( m_data )
         m_data->ref( this );
@@ -54,6 +56,9 @@ Nepomuk::Resource::Resource()
 
 Nepomuk::Resource::Resource( ResourceManager* manager )
 {
+    if(!manager)
+        manager = ResourceManager::instance();
+    QMutexLocker lock( &manager->d->mutex );
     m_data = manager->d->data( QUrl(), QUrl() );
     if ( m_data )
         m_data->ref( this );
@@ -62,6 +67,7 @@ Nepomuk::Resource::Resource( ResourceManager* manager )
 
 Nepomuk::Resource::Resource( const Nepomuk::Resource& res )
 {
+    QMutexLocker lock( &res.m_data->rm()->mutex );
     m_data = res.m_data;
     if ( m_data )
         m_data->ref( this );
@@ -70,6 +76,7 @@ Nepomuk::Resource::Resource( const Nepomuk::Resource& res )
 
 Nepomuk::Resource::Resource( const QString& uri, const QUrl& type )
 {
+    QMutexLocker lock( &ResourceManager::instance()->d->mutex );
     m_data = ResourceManager::instance()->d->data( uri, type );
     if ( m_data )
         m_data->ref( this );
@@ -78,22 +85,29 @@ Nepomuk::Resource::Resource( const QString& uri, const QUrl& type )
 
 Nepomuk::Resource::Resource( const QString& uri, const QUrl& type, ResourceManager* manager )
 {
+    if(!manager)
+        manager = ResourceManager::instance();
+    QMutexLocker lock( &manager->d->mutex );
     m_data = manager->d->data( uri, type );
     if ( m_data )
         m_data->ref( this );
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 Nepomuk::Resource::Resource( const QString& uri, const QString& type )
 {
+    QMutexLocker lock( &ResourceManager::instance()->d->mutex );
     m_data = ResourceManager::instance()->d->data( uri, type );
     if ( m_data )
         m_data->ref( this );
 }
+#endif
 
 
 Nepomuk::Resource::Resource( const QUrl& uri, const QUrl& type )
 {
+    QMutexLocker lock( &ResourceManager::instance()->d->mutex );
     m_data = ResourceManager::instance()->d->data( uri, type );
     if ( m_data )
         m_data->ref( this );
@@ -102,6 +116,9 @@ Nepomuk::Resource::Resource( const QUrl& uri, const QUrl& type )
 
 Nepomuk::Resource::Resource( const QUrl& uri, const QUrl& type, ResourceManager* manager )
 {
+    if(!manager)
+        manager = ResourceManager::instance();
+    QMutexLocker lock( &manager->d->mutex );
     m_data = manager->d->data( uri, type );
     if ( m_data )
         m_data->ref( this );
@@ -110,6 +127,7 @@ Nepomuk::Resource::Resource( const QUrl& uri, const QUrl& type, ResourceManager*
 
 Nepomuk::Resource::Resource( Nepomuk::ResourceData* data )
 {
+    QMutexLocker lock( &data->rm()->mutex );
     m_data = data;
     if ( m_data )
         m_data->ref( this );
@@ -119,6 +137,7 @@ Nepomuk::Resource::Resource( Nepomuk::ResourceData* data )
 Nepomuk::Resource::~Resource()
 {
     if ( m_data ) {
+        QMutexLocker lock(&m_data->rm()->mutex);
         m_data->deref( this );
         if ( m_data->rm()->shouldBeDeleted( m_data ) )
             delete m_data;
@@ -129,6 +148,7 @@ Nepomuk::Resource::~Resource()
 Nepomuk::Resource& Nepomuk::Resource::operator=( const Resource& res )
 {
     if( m_data != res.m_data ) {
+        QMutexLocker lock(&m_data->rm()->mutex);
         if ( m_data && !m_data->deref( this ) && m_data->rm()->shouldBeDeleted( m_data ) ) {
             delete m_data;
         }
@@ -153,10 +173,12 @@ Nepomuk::ResourceManager* Nepomuk::Resource::manager() const
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 QString Nepomuk::Resource::uri() const
 {
     return resourceUri().toString();
 }
+#endif
 
 
 QUrl Nepomuk::Resource::resourceUri() const
@@ -171,10 +193,12 @@ QUrl Nepomuk::Resource::resourceUri() const
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 QString Nepomuk::Resource::type() const
 {
     return resourceType().toString();
 }
+#endif
 
 
 QUrl Nepomuk::Resource::resourceType() const
@@ -226,6 +250,7 @@ QHash<QUrl, Nepomuk::Variant> Nepomuk::Resource::properties() const
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 QHash<QString, Nepomuk::Variant> Nepomuk::Resource::allProperties() const
 {
     QHash<QString, Nepomuk::Variant> pl;
@@ -237,6 +262,7 @@ QHash<QString, Nepomuk::Variant> Nepomuk::Resource::allProperties() const
     }
     return pl;
 }
+#endif
 
 
 bool Nepomuk::Resource::hasProperty( const QUrl& uri ) const
@@ -253,16 +279,20 @@ bool Nepomuk::Resource::hasProperty( const Types::Property& p, const Variant& v
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 bool Nepomuk::Resource::hasProperty( const QString& uri ) const
 {
     return hasProperty( QUrl( uri ) );
 }
+#endif
 
 
+#ifndef KDE_NO_DEPRECATED
 Nepomuk::Variant Nepomuk::Resource::property( const QString& uri ) const
 {
     return property( QUrl( uri ) );
 }
+#endif
 
 
 Nepomuk::Variant Nepomuk::Resource::property( const QUrl& uri ) const
@@ -272,17 +302,18 @@ Nepomuk::Variant Nepomuk::Resource::property( const QUrl& uri ) const
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 void Nepomuk::Resource::setProperty( const QString& uri, const Nepomuk::Variant& value )
 {
     setProperty( QUrl( uri ), value );
 }
+#endif
 
 
 void Nepomuk::Resource::addProperty( const QUrl& uri, const Variant& value )
 {
-    Variant v = property( uri );
-    v.append( value );
-    setProperty( uri, v );
+    determineFinalResourceData();
+    m_data->addProperty( uri, value );
 }
 
 
@@ -293,10 +324,12 @@ void Nepomuk::Resource::setProperty( const QUrl& uri, const Nepomuk::Variant& va
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 void Nepomuk::Resource::removeProperty( const QString& uri )
 {
     removeProperty( QUrl( uri ) );
 }
+#endif
 
 
 void Nepomuk::Resource::removeProperty( const QUrl& uri )
@@ -341,58 +374,59 @@ bool Nepomuk::Resource::isValid() const
 }
 
 
-// KDE 4.5: cache this one in ResourceData
+// TODO: cache this one in ResourceData
 QString Nepomuk::Resource::genericLabel() const
 {
     QString label = this->label();
-    if ( label.isEmpty() ) {
-        label = property( Soprano::Vocabulary::RDFS::label() ).toString();
+    if(!label.isEmpty())
+        return label;
 
-        if ( label.isEmpty() ) {
-            label = property( Nepomuk::Vocabulary::NIE::title() ).toString();
+    label = property( Soprano::Vocabulary::RDFS::label() ).toString();
+    if(!label.isEmpty())
+        return label;
 
-            if ( label.isEmpty() ) {
-                label = property( Nepomuk::Vocabulary::NCO::fullname() ).toString();
+    label = property( Nepomuk::Vocabulary::NIE::title() ).toString();
+    if(!label.isEmpty())
+        return label;
 
-                if ( label.isEmpty() ) {
-                    label = property( Soprano::Vocabulary::NAO::identifier() ).toString();
+    label = property( Nepomuk::Vocabulary::NCO::fullname() ).toString();
+    if(!label.isEmpty())
+        return label;
 
-                    if ( label.isEmpty() ) {
-                        label = property( Soprano::Vocabulary::Xesam::name() ).toString();
+    label = property( Soprano::Vocabulary::NAO::identifier() ).toString();
+    if(!label.isEmpty())
+        return label;
 
-                        if ( label.isEmpty() ) {
-                            label = m_data->pimoThing().label();
+    label = m_data->pimoThing().label();
+    if(!label.isEmpty())
+        return label;
 
-                            if ( label.isEmpty() ) {
-                                label = property( Nepomuk::Vocabulary::NFO::fileName() ).toString();
+    label = property( Nepomuk::Vocabulary::NFO::fileName() ).toString();
+    if(!label.isEmpty())
+        return label;
 
-                                if ( label.isEmpty() ) {
-                                    label = KUrl(property( Nepomuk::Vocabulary::NIE::url() ).toUrl()).fileName();
+    const KUrl nieUrl = property( Nepomuk::Vocabulary::NIE::url() ).toUrl();
+    if(!nieUrl.isEmpty()) {
+        if(nieUrl.isLocalFile())
+            return nieUrl.fileName();
+        else
+            return nieUrl.prettyUrl();
+    }
 
-                                    if ( label.isEmpty() ) {
-                                        QList<Resource> go = property( Vocabulary::PIMO::groundingOccurrence() ).toResourceList();
-                                        if( !go.isEmpty() ) {
-                                            label = go.first().genericLabel();
-                                            if( label == go.first().resourceUri().toString() ) {
-                                                label.clear();
-                                            }
-                                        }
-
-                                        if ( label.isEmpty() ) {
-                                            // ugly fallback
-                                            label = resourceUri().toString();
-                                        }
-                                    }
-                                }
-                            }
-                        }
-                    }
-                }
-            }
+    QList<Resource> go = property( Vocabulary::PIMO::groundingOccurrence() ).toResourceList();
+    if( !go.isEmpty() ) {
+        label = go.first().genericLabel();
+        if( label != KUrl(go.first().resourceUri()).pathOrUrl() ) {
+            return label;
         }
     }
 
-    return label;
+    QString hashValue = property( Vocabulary::NFO::hashValue() ).toString();
+    if( !hashValue.isEmpty() )
+        return hashValue;
+
+    // ugly fallback
+    return KUrl(resourceUri()).pathOrUrl();
 }
 
 
@@ -804,20 +838,55 @@ QString Nepomuk::Resource::ratingUri()
 
 QStringList Nepomuk::Resource::symbols() const
 {
-    return property( Soprano::Vocabulary::NAO::hasSymbol() ).toStringList();
+    QList<Resource> symbolResources = property( Soprano::Vocabulary::NAO::hasSymbol() ).toResourceList();
+
+    QStringList symbolStrings;
+    foreach(const Nepomuk::Resource& symbolRes, symbolResources ) {
+        symbolStrings << symbolRes.label();
+    }
+
+    return symbolStrings;
 }
 
+namespace {
+    QUrl uriForSymbolName(const QString& symbolName) {
+        // Check if it exists
+        // We aren't using Soprano::Node::literalToN3 cause prefLabel has a range of a literal not
+        // of a string
+        QString query = QString::fromLatin1("select ?r where { ?r a %1 . ?r %2 \"%3\" . } LIMIT 1")
+                        .arg( Soprano::Node::resourceToN3(Soprano::Vocabulary::NAO::FreeDesktopIcon()),
+                              Soprano::Node::resourceToN3(Soprano::Vocabulary::NAO::prefLabel()),
+                              symbolName );
+
+        Soprano::Model* model = Nepomuk::ResourceManager::instance()->mainModel();
+        Soprano::QueryResultIterator it = model->executeQuery( query, Soprano::Query::QueryLanguageSparql );
+        if( it.next() ) {
+            return it["r"].uri();
+        }
+        else {
+            Nepomuk::Resource res(QUrl(), Soprano::Vocabulary::NAO::FreeDesktopIcon());
+            res.setLabel( symbolName );
+
+            return res.resourceUri();
+        }
+    }
+}
 
 void Nepomuk::Resource::setSymbols( const QStringList& value )
 {
-    setProperty( Soprano::Vocabulary::NAO::hasSymbol(), Variant( value ) );
+    QList<QUrl> symbolList;
+    foreach( const QString& symbolName, value ) {
+        symbolList << uriForSymbolName(symbolName);
+    }
+
+    setProperty( Soprano::Vocabulary::NAO::hasSymbol(), Variant(symbolList) );
 }
 
 
 void Nepomuk::Resource::addSymbol( const QString& value )
 {
     Variant v = property( Soprano::Vocabulary::NAO::hasSymbol() );
-    v.append( value );
+    v.append( uriForSymbolName(value) );
     setProperty( Soprano::Vocabulary::NAO::hasSymbol(), v );
 }
 
@@ -867,18 +936,38 @@ void Nepomuk::Resource::increaseUsageCount()
 }
 
 
+bool Nepomuk::Resource::isFile() const
+{
+    if( m_data ) {
+        determineFinalResourceData();
+        m_data->load();
+        return m_data->isFile();
+    }
+    else {
+        return false;
+    }
+}
+
+
+Nepomuk::File Nepomuk::Resource::toFile() const
+{
+    return File( *this );
+}
+
+
 // static
 Nepomuk::Resource Nepomuk::Resource::fromResourceUri( const KUrl& uri, const Nepomuk::Types::Class& type, ResourceManager* manager )
 {
     if( !manager )
         manager = ResourceManager::instance();
+    QMutexLocker lock( &manager->d->mutex );
     return Resource( manager->d->dataForResourceUri( uri, type.uri() ) );
 }
 
 
 void Nepomuk::Resource::determineFinalResourceData() const
 {
-    m_data->m_determineUriMutex.lock();
+    QMutexLocker lock( &m_data->rm()->mutex );
 
     // Get an initialized ResourceData instance
     ResourceData* oldData = m_data;
@@ -898,10 +987,6 @@ void Nepomuk::Resource::determineFinalResourceData() const
         }
     }
 
-    // we need to unlock before assigning ourselves to make sure we do not
-    // delete m_data before unlocking
-    oldData->m_determineUriMutex.unlock();
-
     if ( !oldData->cnt() )
         delete oldData;
 }
